pub struct TextWrapping {
pub max_width: f32,
pub max_rows: usize,
pub break_anywhere: bool,
pub overflow_character: Option<char>,
}Fields
max_width: f32Try to break text so that no row is wider than this.
Set to f32::INFINITY to turn off wrapping.
Note that \n always produces a new line.
max_rows: usizeMaximum amount of rows the text should have.
Set to 0 to disable this.
break_anywhere: boolDon’t try to break text at an appropriate place.
overflow_character: Option<char>Character to use to represent clipped text, … for example, which is the default.