pub struct PointerState { /* private fields */ }Expand description
Mouse or touch state.
Implementations
Sourceimpl PointerState
impl PointerState
Sourcepub fn press_origin(&self) -> Option<Pos2>
pub fn press_origin(&self) -> Option<Pos2>
Where did the current click/drag originate?
None if no mouse button is down.
Sourcepub fn press_start_time(&self) -> Option<f64>
pub fn press_start_time(&self) -> Option<f64>
When did the current click/drag originate?
None if no mouse button is down.
Sourcepub fn hover_pos(&self) -> Option<Pos2>
pub fn hover_pos(&self) -> Option<Pos2>
If it is a good idea to show a tooltip, where is pointer?
Sourcepub fn interact_pos(&self) -> Option<Pos2>
pub fn interact_pos(&self) -> Option<Pos2>
If you detect a click or drag and wants to know where it happened, use this.
Latest position of the mouse, but ignoring any Event::PointerGone
if there were interactions this frame.
When tapping a touch screen, this will be the location of the touch.
Sourcepub fn has_pointer(&self) -> bool
pub fn has_pointer(&self) -> bool
Do we have a pointer?
false if the mouse is not over the egui area, or if no touches are down on touch screens.
Sourcepub fn is_still(&self) -> bool
pub fn is_still(&self) -> bool
Is the pointer currently still?
This is smoothed so a few frames of stillness is required before this returns true.
Sourcepub fn is_moving(&self) -> bool
pub fn is_moving(&self) -> bool
Is the pointer currently moving?
This is smoothed so a few frames of stillness is required before this returns false.
Sourcepub fn any_pressed(&self) -> bool
pub fn any_pressed(&self) -> bool
Was any pointer button pressed (!down -> down) this frame?
This can sometimes return true even if any_down() == false
because a press can be shorted than one frame.
Sourcepub fn any_released(&self) -> bool
pub fn any_released(&self) -> bool
Was any pointer button released (down -> !down) this frame?
Is this button currently down?
Sourcepub fn primary_down(&self) -> bool
pub fn primary_down(&self) -> bool
Is the primary button currently down?
Sourcepub fn secondary_down(&self) -> bool
pub fn secondary_down(&self) -> bool
Is the secondary button currently down?
Sourcepub fn middle_down(&self) -> bool
pub fn middle_down(&self) -> bool
Is the middle button currently down?